unit ListSerFm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls;

type
  TListServerForm = class(TForm)
    Edit1: TEdit;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Tree: TTreeView;
    procedure Edit1Change(Sender: TObject);
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
    procedure FormShow(Sender: TObject);
    procedure TreeDblClick(Sender: TObject);
  private
    function EnumNet( PRoot: PNetResource; Node: TTreeNode ): Boolean;
  public
    Domain : String;
    Server : String;
    SQLServersOnly : boolean;
  end;

var
  ListServerForm: TListServerForm;

implementation

{$R *.DFM}


procedure TListServerForm.Edit1Change(Sender: TObject);
begin
   OkBtn.Enabled := Edit1.Text <> '';
end;

procedure TListServerForm.TreeChange(Sender: TObject; Node: TTreeNode);
begin
   Edit1.Text := Node.Text;
   if Node.HasChildren then
   begin
     Domain := Node.Text;
     Server := '';
   end
   else
   begin
     Server := Node.Text;
     Domain := '';
   end;
end;

procedure TListServerForm.FormShow(Sender: TObject);
var
  FirstNode : TTreeNode;
begin
   Screen.Cursor := crHourGlass;
   Tree.Items.Clear;
   EnumNet( nil, Tree.Items.GetFirstNode );
   Tree.FullExpand;
   FirstNode := Tree.Items.GetFirstNode;
   if FirstNode <> nil then
      FirstNode.MakeVisible;
   Screen.Cursor := crArrow;
end;

function TListServerForm.EnumNet( PRoot: PNetResource; Node: TTreeNode ): Boolean;
var
   NewNode: TTreeNode;
   info   : array[0..40] of TNetResource;
   phEnum : THandle;
   Count, size    : DWORD;
   status, i  : DWORD;
begin
  if WNetOpenEnum( RESOURCE_GLOBALNET, RESOURCETYPE_DISK,
     RESOURCEUSAGE_CONTAINER, PRoot, phEnum ) = NO_ERROR then
  begin
    repeat
      Count := -1;
      size := sizeof( info );
      status := WNetEnumResource( phEnum, Count, @info[0], size );
      for i:=0 to Count-1 do
      begin
        case info[i].dwDisplayType of
          RESOURCEDISPLAYTYPE_DOMAIN:
            NewNode := Tree.Items.Add( Node, StrPas(info[i].lpRemoteName) );
          RESOURCEDISPLAYTYPE_SERVER:
            NewNode := Tree.Items.AddChild( Node, Copy(StrPas(info[i].lpRemoteName),3,255) );
          else
            NewNode := Node;
        end;
        EnumNet( @info[i], NewNode );
      end;
    until status = ERROR_NO_MORE_ITEMS;
    WNetCloseEnum( phEnum );
  end;
  result := True;
end;

{
function TListServerForm.EnumNet( Node: TTreeNode ): Boolean;
var
   NewNode: TTreeNode;
   info, iinfo  : PSERVER_INFO_100;
   s   :  String;
   i, j   :  Integer;
   info1 : PWKSTA_INFO_100;
   Server, Domain : LPWSTR;
   ent, tot, ResumeHandle : DWORD;
   ent1, tot1, Resume1 : DWORD;
   ServerType : integer;
begin
  if SQLServersOnly then ServerType := 4
  else ServerType := 1;
  GetMem( Server, 50 );
  GetMem( Domain, 50 );
  NetWkstaGetInfo( nil, 100, PBYTE(info1) );
  s := WideCharToString( info1^.wksi100_computername );
  StringToWideChar( '\\'+s, Server, 50 );
  ResumeHandle := 0;
  s := WideCharToString( info1^.wksi100_langroup );
  StringToWideChar( s, Domain, 50 );

  NetServerEnum( Server, 100, PBYTE(info), sizeof(info),
       ent, tot, $80000000, Domain, ResumeHandle );
  for i:=0 to ent-1 do
  begin
    NewNode := Tree.Items.Add( Node, WideCharToString(info^.sv101_name) );
    NetServerEnum( Server, 100, PBYTE(iinfo), sizeof(iinfo),
       ent1, tot1, ServerType, info^.sv101_name, Resume1 );
    for j:=0 to ent1-1 do
    begin
       Tree.Items.AddChild( NewNode, WideCharToString(iinfo^.sv101_name) );
       Inc( iinfo );
    end;
    while ent1 <> tot1 do
    begin
       NetServerEnum( Server, 100, PBYTE(iinfo), sizeof(iinfo),
          ent1, tot1, ServerType, info^.sv101_name, Resume1 );
       for j:=0 to ent1-1 do
       begin
          Tree.Items.AddChild( NewNode, WideCharToString(iinfo^.sv101_name) );
          Inc( iinfo );
       end;
    end;
    Inc(info);
  end;
  FreeMem( Server, 50 );
  FreeMem( Domain, 50 );
  Result := True;
end;
}

procedure TListServerForm.TreeDblClick(Sender: TObject);
begin
   SendMessage( OKBtn.Handle, BM_CLICK, 1, 0 );
end;

end.
